@@ -1,5 +1,4 @@
 //go:build darwin
-// +build darwin
 
 /*
 Copyright 2021, Red Hat, Inc - All rights reserved.
@@ -35,6 +34,7 @@ import (
 	"github.com/Code-Hex/vz/v3"
 	"github.com/crc-org/vfkit/pkg/cmdline"
 	"github.com/crc-org/vfkit/pkg/config"
+	"github.com/crc-org/vfkit/pkg/process"
 	"github.com/crc-org/vfkit/pkg/rest"
 	restvf "github.com/crc-org/vfkit/pkg/rest/vf"
 	"github.com/crc-org/vfkit/pkg/vf"
@@ -72,15 +72,18 @@ func newVMConfiguration(opts *cmdline.Options) (*config.VirtualMachine, error) {
 		return nil, err
 	}
 
-	log.Info(opts)
-	log.Infof("boot parameters: %+v", bootloader)
-	log.Info()
+	log.Debugf("parsed options: %+v", opts)
+	log.Debugf("boot parameters: %+v", bootloader)
 
 	vmConfig := config.NewVirtualMachine(
 		opts.Vcpus,
 		uint64(opts.MemoryMiB),
 		bootloader,
 	)
+	if opts.Nested && !vz.IsNestedVirtualizationSupported() {
+		return nil, fmt.Errorf("nested virtualization is not supported")
+	}
+	vmConfig.Nested = opts.Nested
 	log.Info("virtual machine parameters:")
 	log.Infof("\tvCPUs: %d", opts.Vcpus)
 	log.Infof("\tmemory: %d MiB", opts.MemoryMiB)
@@ -100,14 +103,52 @@ func newVMConfiguration(opts *cmdline.Options) (*config.VirtualMachine, error) {
 		opts.Devices = append(opts.Devices, fmt.Sprintf("virtio-blk,path=%s", cloudInitISO))
 	}
 
+	if opts.PidFile != "" {
+		execPath, err := os.Executable()
+		if err != nil {
+			return nil, fmt.Errorf("could not determine executable path: %w", err)
+		}
+		vfProcess := process.New(os.Args[0], opts.PidFile, execPath)
+		pid := os.Getpid()
+		err = vfProcess.WritePidFile(pid)
+		if err != nil {
+			return nil, fmt.Errorf("could not write PID: %w", err)
+		}
+	}
+
 	if err := vmConfig.AddDevicesFromCmdLine(opts.Devices); err != nil {
 		return nil, err
 	}
 
+	if opts.UseGUI {
+		if len(vmConfig.VirtioGPUDevices()) == 0 {
+			log.Warnf("--gui flag specified but no virtio-gpu device configured, automatically adding it")
+			dev, err := config.VirtioGPUNew()
+			if err != nil {
+				return nil, fmt.Errorf("failed to add virtio-gpu device: %w", err)
+			}
+			dev.(*config.VirtioGPU).UsesGUI = true
+			err = vmConfig.AddDevice(dev)
+			if err != nil {
+				return nil, fmt.Errorf("failed to add virtio-gpu device: %w", err)
+			}
+		}
+		if len(vmConfig.VirtioInputDevices()) == 0 {
+			log.Warnf("--gui flag specified but no virtio-input device configured, automatically adding it")
+			dev, err := config.VirtioInputNew(config.VirtioInputKeyboardDevice)
+			if err != nil {
+				return nil, fmt.Errorf("failed to add virtio-input device: %w", err)
+			}
+			err = vmConfig.AddDevice(dev)
+			if err != nil {
+				return nil, fmt.Errorf("failed to add virtio-input device: %w", err)
+			}
+		}
+	}
+
 	if err := vmConfig.AddIgnitionFileFromCmdLine(opts.IgnitionPath); err != nil {
 		return nil, fmt.Errorf("failed to add ignition file: %w", err)
 	}
-
 	return vmConfig, nil
 }
 
@@ -136,8 +177,6 @@ func runVFKit(vmConfig *config.VirtualMachine, opts *cmdline.Options) error {
 	runtime.LockOSThread()
 	defer runtime.UnlockOSThread()
 
-	util.SetupExitSignalHandling()
-
 	gpuDevs := vmConfig.VirtioGPUDevices()
 	if opts.UseGUI && len(gpuDevs) > 0 {
 		gpuDevs[0].UsesGUI = true
@@ -157,19 +196,33 @@ func runVFKit(vmConfig *config.VirtualMachine, opts *cmdline.Options) error {
 		}
 		srv.Start()
 	}
+
+	shutdownFunc := func() {
+		log.Debugf("shutting down...")
+		stopped, err := vfVM.RequestStop()
+		if err != nil {
+			log.Errorf("failed to shutdown VM: %v", err)
+		} else if !stopped {
+			log.Warnf("VM did not acknowledge stop request")
+		}
+		if err := waitForVMState(vfVM, vz.VirtualMachineStateStopped, time.After(5*time.Second)); err != nil {
+			log.Warnf("failed to wait for VM stop: %v, forcing stop", err)
+			if forceErr := vfVM.Stop(); forceErr != nil {
+				log.Errorf("failed to force stop VM: %v", forceErr)
+			}
+		} else {
+			log.Debugf("VM stopped gracefully")
+		}
+
+	}
+	util.SetupExitSignalHandling(shutdownFunc)
 	return runVirtualMachine(vmConfig, vfVM)
 }
 
 func runVirtualMachine(vmConfig *config.VirtualMachine, vm *vf.VirtualMachine) error {
 	if vm.Config().Ignition != nil {
 		go func() {
-			file, err := os.Open(vmConfig.Ignition.ConfigPath)
-			if err != nil {
-				log.Error(err)
-			}
-			defer file.Close()
-			reader := file
-			if err := startIgnitionProvisionerServer(reader, vmConfig.Ignition.SocketPath); err != nil {
+			if err := startIgnitionProvisionerServer(vm, vmConfig.Ignition.ConfigPath, vmConfig.Ignition.VsockPort); err != nil {
 				log.Error(err)
 			}
 			log.Debug("ignition vsock server exited")
@@ -190,7 +243,7 @@ func runVirtualMachine(vmConfig *config.VirtualMachine, vm *vf.VirtualMachine) e
 		port := vsock.Port
 		socketURL := vsock.SocketURL
 		if socketURL == "" {
-			// the timesync code adds a vsock device without an associated URL.
+			// timesync and ignition add a vsock device without an associated URL.
 			continue
 		}
 		var listenStr string
@@ -243,23 +296,21 @@ func runVirtualMachine(vmConfig *config.VirtualMachine, vm *vf.VirtualMachine) e
 	return <-errCh
 }
 
-func startIgnitionProvisionerServer(ignitionReader io.Reader, ignitionSocketPath string) error {
-	mux := http.NewServeMux()
-	mux.HandleFunc("/", func(w http.ResponseWriter, _ *http.Request) {
-		_, err := io.Copy(w, ignitionReader)
-		if err != nil {
-			log.Errorf("failed to serve ignition file: %v", err)
-		}
-	})
-
-	listener, err := net.Listen("unix", ignitionSocketPath)
+func startIgnitionProvisionerServer(vm *vf.VirtualMachine, configPath string, vsockPort uint32) error {
+	ignitionReader, err := os.Open(configPath)
 	if err != nil {
 		return err
 	}
+	defer ignitionReader.Close()
 
-	util.RegisterExitHandler(func() {
-		os.Remove(ignitionSocketPath)
-	})
+	vsockDevices := vm.SocketDevices()
+	if len(vsockDevices) != 1 {
+		return fmt.Errorf("VM has too many/not enough virtio-vsock devices (%d)", len(vsockDevices))
+	}
+	listener, err := vsockDevices[0].Listen(vsockPort)
+	if err != nil {
+		return err
+	}
 
 	defer func() {
 		if err := listener.Close(); err != nil {
@@ -267,14 +318,23 @@ func startIgnitionProvisionerServer(ignitionReader io.Reader, ignitionSocketPath
 		}
 	}()
 
+	return startIgnitionProvisionerServerInternal(ignitionReader, listener)
+}
+
+func startIgnitionProvisionerServerInternal(ignitionReader io.ReadSeeker, listener net.Listener) error {
+	mux := http.NewServeMux()
+	mux.HandleFunc("/", func(w http.ResponseWriter, req *http.Request) {
+		http.ServeContent(w, req, "", time.Time{}, ignitionReader)
+	})
+
 	srv := &http.Server{
 		Handler:           mux,
-		Addr:              ignitionSocketPath,
+		Addr:              listener.Addr().String(),
 		ReadHeaderTimeout: 10 * time.Second,
 		IdleTimeout:       60 * time.Second,
 	}
 
-	log.Debugf("ignition socket: %s", ignitionSocketPath)
+	log.Debugf("ignition socket: %s", listener.Addr().String())
 	return srv.Serve(listener)
 }
 
@@ -289,8 +349,9 @@ func generateCloudInitImage(files []string) (string, error) {
 	}
 
 	configFiles := map[string]io.Reader{
-		"user-data": nil,
-		"meta-data": nil,
+		"user-data":      nil,
+		"meta-data":      nil,
+		"network-config": nil,
 	}
 
 	hasConfigFile := false
@@ -306,7 +367,9 @@ func generateCloudInitImage(files []string) (string, error) {
 
 		filename := filepath.Base(path)
 		if _, ok := configFiles[filename]; ok {
-			hasConfigFile = true
+			if filename == "user-data" || filename == "meta-data" {
+				hasConfigFile = true
+			}
 			configFiles[filename] = file
 		}
 	}
